import { useState, useEffect, useRef, useCallback, memo } from 'react';
import { motion } from 'framer-motion';
import './ChatDemo.css';

// Named function component to fix ESLint error
function ChatDemo() {
  // Indicate to the browser we'll have animations
  const supportsPerformance = 'requestIdleCallback' in window;
  
  // Gradient animation ref
  const gradientRef = useRef<HTMLDivElement>(null);
  
  // Chat message display states
  const [showFirstNovaMessage, setShowFirstNovaMessage] = useState(false);
  const [showFirstMichaelMessage, setShowFirstMichaelMessage] = useState(false);
  const [showSecondNovaMessage, setShowSecondNovaMessage] = useState(false);
  const [showTimeSkip, setShowTimeSkip] = useState(false);
  const [showThirdNovaMessage, setShowThirdNovaMessage] = useState(false);
  const [showSecondMichaelMessage, setShowSecondMichaelMessage] = useState(false);
  const [showFinalMessage, setShowFinalMessage] = useState(false);
  
  // Typing indicator states
  const [novaTyping, setNovaTyping] = useState(true); // Start with Nova typing
  const [michaelTyping, setMichaelTyping] = useState(false);
  
  // UI enhancement states
  const [highlightStory, setHighlightStory] = useState(false);
  const [showRewind, setShowRewind] = useState(false);
  
  // Track demo resets
  const [resetCount, setResetCount] = useState(0);
  
  // Ref for auto-scrolling
  const chatBodyRef = useRef<HTMLDivElement>(null);
  
  // Track animation timers for cleanup
  const timerRefs = useRef<NodeJS.Timeout[]>([]);
  
  const messages = [
    { sender: 'Nova', content: "Hey Michael, I think you should meet Elena this Wednesday. She's an urban planner who builds community spaces the way you build apps—focusing on how people move through systems." },
    { sender: 'Michael', content: "A city planner? That's not what I expected... but tell me more!" },
    { sender: 'Nova', content: "I get it - it sounds offbeat. But our data shows you two have a 92% match. You both love systems thinking and retrofuturism. Remember when you mentioned Blade Runner? Elena's designing a park inspired by it!" },
    { sender: 'Nova', content: "Hey Michael, how was your call with Elena? Did you dive into the Blade Runner park idea?" },
    { sender: 'Michael', content: "Yeah, it was one of the best calls I've had in months. Her take on using game theory in physical spaces really shifted how I think about app design. We're meeting again next week to brainstorm more." },
  ];

  const finalMessage = { 
    sender: 'Nova', 
    content: "That's exactly what I was hoping for! And guess what - I've already lined up your next match. Can't wait for you to meet who I've arranged for you next!" 
  };

  // Reset the demo to initial state - memoized with useCallback
  const resetDemo = useCallback(() => {
    // Clear all existing timers
    timerRefs.current.forEach(timer => clearTimeout(timer));
    timerRefs.current = [];
    
    // Reset all message display states
    setShowFirstNovaMessage(false);
    setShowFirstMichaelMessage(false);
    setShowSecondNovaMessage(false);
    setShowTimeSkip(false);
    setShowThirdNovaMessage(false);
    setShowSecondMichaelMessage(false);
    setShowFinalMessage(false);
    
    // Reset typing states
    setNovaTyping(true);
    setMichaelTyping(false);
    
    // Reset UI enhancement states
    setHighlightStory(false);
    setShowRewind(false);
    
    // Increment reset count to trigger animation sequence
    setResetCount(prevCount => prevCount + 1);
  }, []);
  
  // Auto-scroll to bottom when new messages appear
  useEffect(() => {
    if (chatBodyRef.current) {
      chatBodyRef.current.scrollTop = chatBodyRef.current.scrollHeight;
    }
  }, [
    showFirstNovaMessage, 
    showFirstMichaelMessage, 
    showSecondNovaMessage, 
    showTimeSkip,
    showThirdNovaMessage,
    showSecondMichaelMessage,
    showFinalMessage,
    novaTyping, 
    michaelTyping
  ]);

  // Helper function to add a timer and keep track of it - memoized with useCallback
  const addTimer = useCallback((callback: () => void, delay: number) => {
    // Use requestAnimationFrame for short delays when possible
    if (delay < 100 && 'requestAnimationFrame' in window) {
      const start = performance.now();
      const handle = requestAnimationFrame(function frameFunc(timestamp) {
        if (timestamp - start < delay) {
          const rafId = requestAnimationFrame(frameFunc);
          // Store RAF id so we can cancel it if needed
          timerRefs.current.push(setTimeout(() => cancelAnimationFrame(rafId), 0));
          return;
        }
        callback();
      });
      // Store a way to cancel the animation frame
      timerRefs.current.push(setTimeout(() => cancelAnimationFrame(handle), 0));
      return handle;
    } else {
      // Use setTimeout for longer delays
      const timer = setTimeout(callback, delay);
      timerRefs.current.push(timer);
      return timer;
    }
  }, []);

  // Sequential animation - triggered on first render and after reset  
  useEffect(() => {
    // Use requestIdleCallback to wait until the browser has free time
    const startAnimations = () => {
      // Clear any existing timers
      timerRefs.current.forEach(timer => clearTimeout(timer));
      timerRefs.current = [];
      
      // Initial sequence - Nova types first message
      addTimer(() => {
        setNovaTyping(false); // Stop Nova's typing indicator
        setShowFirstNovaMessage(true); // Show Nova's first message
        
        // After a pause, Michael starts typing
        addTimer(() => {
          setMichaelTyping(true); // Show Michael's typing indicator
          
          // After typing time, show Michael's message
          addTimer(() => {
            setMichaelTyping(false); // Stop Michael's typing indicator
            setShowFirstMichaelMessage(true); // Show Michael's message
            
            // After a pause, Nova starts typing again
            addTimer(() => {
              setNovaTyping(true); // Show Nova's typing indicator
              
              // After typing time, show Nova's second message
              addTimer(() => {
                setNovaTyping(false); // Stop Nova's typing indicator
                setShowSecondNovaMessage(true); // Show Nova's second message
                
                // Turn off typing before time skip
                setNovaTyping(false);
                
                // After a pause, show time skip and final sequence
                addTimer(() => {
                  setShowTimeSkip(true); // Show "A few days later..."
                  
                  // After time skip, wait a moment before Nova starts typing
                  addTimer(() => {
                    setNovaTyping(true);
                    
                    // After typing time, show Nova's third message
                    addTimer(() => {
                      setNovaTyping(false);
                      setShowThirdNovaMessage(true);
                      
                      // After third Nova message, wait a moment before Michael starts typing
                      addTimer(() => {
                        setMichaelTyping(true);
                        
                        addTimer(() => {
                          setMichaelTyping(false);
                          setShowSecondMichaelMessage(true);
                        
                          // Finally, show Nova's final message
                          addTimer(() => {
                            setNovaTyping(true);
                            
                            addTimer(() => {
                              setNovaTyping(false);
                              setShowFinalMessage(true);
                              
                              // After showing final message, highlight it
                              addTimer(() => {
                                setHighlightStory(true); // Add highlight to final message
                                
                                // After highlighting, show rewind button
                                addTimer(() => {
                                  setShowRewind(true); // Show rewind button
                                }, 1000);
                              }, 800);
                            }, 1500);
                          }, 800);
                        }, 1500);
                      }, 1500);
                    }, 1500);
                  }, 800);
                }, 1500);
              }, 800);
            }, 1500);
          }, 800);
        }, 1500);
      }, 1000);
    };
    
    // Use requestIdleCallback if supported, otherwise just start animations
    if ('requestIdleCallback' in window && supportsPerformance) {
      requestIdleCallback(() => startAnimations());
    } else {
      startAnimations();
    }
    
    // Cleanup function to clear all timers when component unmounts or resets
    return () => {
      timerRefs.current.forEach(timer => clearTimeout(timer));
    };
  }, [resetCount, addTimer, supportsPerformance]); // Depend on resetCount to retrigger sequence after reset

  // Add keyframes for animations
  useEffect(() => {
    const styleElement = document.createElement('style');
    styleElement.innerHTML = `
      @keyframes pulse-glow {
        0% {
          opacity: 0.7;
          transform: scale(0.95);
        }
        50% {
          opacity: 1;
        }
        100% {
          opacity: 0.8;
          transform: scale(1.05);
        }
      }
      
      @keyframes pulse-glow-secondary {
        0% {
          opacity: 0.6;
          transform: translateZ(0) scale(1.1);
        }
        50% {
          opacity: 0.9;
        }
        100% {
          opacity: 0.7;
          transform: translateZ(0) scale(1.2);
        }
      }
      
      @keyframes pulse-glow-tertiary {
        0% {
          opacity: 0.6;
          transform: translateY(-5px);
        }
        50% {
          opacity: 0.9;
        }
        100% {
          opacity: 0.8;
          transform: translateY(5px);
        }
      }
      
      @keyframes pulse-rotate {
        0% {
          transform: translateZ(0) rotate(0deg);
        }
        100% {
          transform: translateZ(0) rotate(360deg);
        }
      }
      
      @keyframes shimmer {
        0% {
          transform: translateX(-100%);
        }
        100% {
          transform: translateX(100%);
        }
      }
      
      @keyframes float {
        0% {
          transform: translateY(0px);
        }
        50% {
          transform: translateY(-10px);
        }
        100% {
          transform: translateY(0px);
        }
      }
      
      .animate-shimmer {
        animation: shimmer 3s infinite;
      }
    `;
    document.head.appendChild(styleElement);
    
    return () => {
      document.head.removeChild(styleElement);
    };
  }, []);

  return (
    <div className="chat-demo-wrapper" style={{ width: '100%', maxWidth: '420px', minHeight: '420px' }}>
      {/* Main outer glow effect - positioned outside the chat container */}
      <div 
        ref={gradientRef}
        className="absolute -inset-10 sm:-inset-16 -z-10"
        style={{
          background: 'radial-gradient(ellipse at 50% 50%, rgba(135, 76, 255, 0.9) 0%, rgba(92, 21, 208, 0.4) 40%, rgba(20, 0, 80, 0) 70%)',
          filter: 'blur(30px)',
          transform: 'translateZ(0)',
          willChange: 'transform, opacity',
          animation: 'pulse-glow 8s ease-in-out infinite alternate'
        }}
      ></div>
      
      {/* Secondary outer glow for depth - dramatically larger than container */}
      <div 
        className="absolute -inset-14 sm:-inset-24 -z-20"
        style={{
          background: 'radial-gradient(ellipse at 40% 40%, rgba(143, 58, 255, 0.7) 0%, rgba(66, 22, 150, 0.3) 50%, transparent 80%)',
          filter: 'blur(45px)',
          transform: 'translateZ(0) scale(1.1)',
          animation: 'pulse-glow-secondary 10s ease-in-out infinite alternate'
        }}
      ></div>
      
      {/* Third layer glow with rotation - creates movement in the effect */}
      <div 
        className="absolute -inset-12 sm:-inset-20 -z-30"
        style={{
          background: 'conic-gradient(from 225deg at 50% 50%, rgba(159, 122, 255, 0.7) 0deg, rgba(95, 40, 200, 0.2) 120deg, rgba(76, 15, 150, 0.1) 240deg, rgba(130, 80, 230, 0.5) 360deg)',
          filter: 'blur(50px)',
          transform: 'translateZ(0)',
          opacity: 0.5,
          animation: 'pulse-rotate 25s linear infinite'
        }}
      ></div>
      
      {/* Fourth layer - adds blue tint as seen in the screenshot */}
      <div 
        className="absolute -inset-10 sm:-inset-16 -z-40"
        style={{
          background: 'radial-gradient(circle at 30% 70%, rgba(70, 120, 255, 0.6) 0%, rgba(40, 80, 210, 0.3) 50%, transparent 80%)',
          filter: 'blur(45px)',
          opacity: 0.6,
          animation: 'pulse-glow 12s ease-in-out infinite alternate-reverse'
        }}
      ></div>
      
      {/* Fifth layer - extra highlight for dramatic effect */}
      <div 
        className="absolute -inset-8 -top-12 sm:-inset-12 sm:-top-16 -z-50"
        style={{
          background: 'radial-gradient(circle at 50% 0%, rgba(180, 100, 255, 0.8) 0%, rgba(120, 40, 220, 0.3) 40%, transparent 70%)',
          filter: 'blur(30px)',
          opacity: 0.7,
          animation: 'pulse-glow-tertiary 15s ease-in-out infinite alternate'
        }}
      ></div>
      
      <div className="chat-container bg-gray-900/90 rounded-xl overflow-hidden w-full relative shadow-2xl shadow-purple-900/50 border border-indigo-500/40 backdrop-blur-sm z-10" style={{ minHeight: '380px' }}>
        {/* Rewind button - appears after conversation completes */}
        {showRewind && (
          <motion.button
            initial={{ opacity: 0, scale: 0.8 }}
            animate={{ opacity: 1, scale: 1 }}
            transition={{ delay: 0.5 }}
            onClick={resetDemo}
            className="absolute top-3 right-3 z-20 w-8 h-8 rounded-full bg-indigo-600 hover:bg-indigo-700 flex items-center justify-center text-white hover:rotate-180 transition-all duration-300"
            title="Replay conversation"
          >
            <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round">
              <path d="M3 2v6h6"></path>
              <path d="M3 13a9 9 0 1 0 3-7.7L3 8"></path>
            </svg>
          </motion.button>
        )}
        
        <div className="chat-header bg-gradient-to-r from-indigo-600 to-purple-700 p-4 flex items-center gap-3 relative overflow-hidden">
          {/* Header shine effect */}
          <div className="absolute inset-0 opacity-20 bg-gradient-to-r from-transparent via-white/40 to-transparent -translate-x-full animate-shimmer"></div>
          
          {/* Additional glow for header */}
          <div className="absolute inset-0 bg-gradient-to-b from-white/5 to-transparent"></div>
          
          <div className="w-10 h-10 rounded-full bg-white/10 flex items-center justify-center">
            <span className="text-white font-bold">N</span>
          </div>
          <div>
            <h3 className="font-semibold text-white">Nova</h3>
            <p className="text-xs text-white/70">Your AI Social Strategist</p>
          </div>
        </div>
        
        <div 
          ref={chatBodyRef}
          className="chat-body p-4 space-y-4 min-h-[300px] sm:min-h-[350px] max-h-[350px] sm:max-h-[500px] overflow-y-auto scrollbar-hide"
        >
          {/* First Nova message */}
          {showFirstNovaMessage && (
            <div className="message flex justify-start">
              <div className="w-8 h-8 rounded-full bg-indigo-600 flex items-center justify-center mr-2 self-end">
                <span className="text-white text-xs font-bold">N</span>
                
              </div>
              <div className="message-content max-w-[75%]">
                <div className="message-bubble bg-indigo-900/40 text-white rounded-tr-xl rounded-br-xl rounded-bl-xl p-3 mt-1 shadow-md shadow-indigo-900/10">
                  {messages[0].content}
                </div>
              </div>
            </div>
          )}
          
          {/* First Michael message */}
          {showFirstMichaelMessage && (
            <div className="message flex justify-end">
              <div className="message-content max-w-[75%]">
                <div className="message-bubble bg-indigo-600 text-white rounded-tl-xl rounded-bl-xl rounded-br-xl p-3 mt-1 shadow-md shadow-indigo-900/10">
                  {messages[1].content}
                </div>
              </div>
              <div className="w-8 h-8 rounded-full bg-gray-600 flex items-center justify-center ml-2 self-end">
                <span className="text-white text-xs font-bold">M</span>
              </div>
            </div>
          )}
          
          {/* Second Nova message */}
          {showSecondNovaMessage && (
            <div className="message flex justify-start">
              <div className="w-8 h-8 rounded-full bg-indigo-600 flex items-center justify-center mr-2 self-end">
                <span className="text-white text-xs font-bold">N</span>
              </div>
              <div className="message-content max-w-[75%]">
                <div className="message-bubble bg-indigo-900/40 text-white rounded-tr-xl rounded-br-xl rounded-bl-xl p-3 mt-1 shadow-md shadow-indigo-900/10">
                  {messages[2].content}
                </div>
              </div>
            </div>
          )}
          
          {/* Michael typing indicator for first part of conversation */}
          {michaelTyping && !showTimeSkip && !showThirdNovaMessage && (
            <div className="message flex justify-end">
              <div className="message-content">
                <div className="message-bubble bg-indigo-600 rounded-tl-xl rounded-bl-xl rounded-br-xl p-3 mt-1 flex items-center shadow-md shadow-indigo-900/10">
                  <div className="typing-indicator flex gap-1">
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full"></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.2s" }}></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.4s" }}></span>
                  </div>
                </div>
              </div>
              <div className="w-8 h-8 rounded-full bg-gray-600 flex items-center justify-center ml-2 self-end">
                <span className="text-white text-xs font-bold">M</span>
              </div>
            </div>
          )}
          
          {/* Nova typing indicator for first part of conversation */}
          {novaTyping && !showTimeSkip && !showSecondNovaMessage && (
            <div className="message flex justify-start">
              <div className="w-8 h-8 rounded-full bg-indigo-600 flex items-center justify-center mr-2 self-end">
                <span className="text-white text-xs font-bold">N</span>
              </div>
              <div className="message-content">
                <div className="message-bubble bg-indigo-900/40 rounded-tr-xl rounded-br-xl rounded-bl-xl p-3 mt-1 flex items-center shadow-md shadow-indigo-900/10">
                  <div className="typing-indicator flex gap-1">
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full"></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.2s" }}></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.4s" }}></span>
                  </div>
                </div>
              </div>
            </div>
          )}
          
          {/* Time skip message */}
          {showTimeSkip && (
            <div className="time-badge text-xs font-medium bg-gray-800/50 py-1 px-4 rounded-full mx-auto w-fit text-indigo-200/90">
              A few days later...
            </div>
          )}
          
          {/* Nova typing indicator for second part of conversation (after time skip) */}
          {novaTyping && showTimeSkip && !showThirdNovaMessage && (
            <div className="message flex justify-start">
              <div className="w-8 h-8 rounded-full bg-indigo-600 flex items-center justify-center mr-2 self-end">
                <span className="text-white text-xs font-bold">N</span>
              </div>
              <div className="message-content">
                <div className="message-bubble bg-indigo-900/40 rounded-tr-xl rounded-br-xl rounded-bl-xl p-3 mt-1 flex items-center shadow-md shadow-indigo-900/10">
                  <div className="typing-indicator flex gap-1">
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full"></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.2s" }}></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.4s" }}></span>
                  </div>
                </div>
              </div>
            </div>
          )}
          
          {/* Third Nova message */}
          {showThirdNovaMessage && (
            <div className="message flex justify-start">
              <div className="w-8 h-8 rounded-full bg-indigo-600 flex items-center justify-center mr-2 self-end">
                <span className="text-white text-xs font-bold">N</span>
              </div>
              <div className="message-content max-w-[75%]">
                <div className="message-bubble bg-indigo-900/40 text-white rounded-tr-xl rounded-br-xl rounded-bl-xl p-3 mt-1 shadow-md shadow-indigo-900/10">
                  {messages[3].content}
                </div>
              </div>
            </div>
          )}
          
          {/* Michael typing indicator for second part of conversation (after time skip) */}
          {michaelTyping && showTimeSkip && showThirdNovaMessage && !showSecondMichaelMessage && (
            <div className="message flex justify-end">
              <div className="message-content">
                <div className="message-bubble bg-indigo-600 rounded-tl-xl rounded-bl-xl rounded-br-xl p-3 mt-1 flex items-center shadow-md shadow-indigo-900/10">
                  <div className="typing-indicator flex gap-1">
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full"></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.2s" }}></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.4s" }}></span>
                  </div>
                </div>
              </div>
              <div className="w-8 h-8 rounded-full bg-gray-600 flex items-center justify-center ml-2 self-end">
                <span className="text-white text-xs font-bold">M</span>
              </div>
            </div>
          )}
          
          {/* Second Michael message */}
          {showSecondMichaelMessage && (
            <div className="message flex justify-end">
              <div className="message-content max-w-[75%]">
                <div className="message-bubble bg-indigo-600 text-white rounded-tl-xl rounded-bl-xl rounded-br-xl p-3 mt-1 shadow-md shadow-indigo-900/10">
                  {messages[4].content}
                </div>
              </div>
              <div className="w-8 h-8 rounded-full bg-gray-600 flex items-center justify-center ml-2 self-end">
                <span className="text-white text-xs font-bold">M</span>
              </div>
            </div>
          )}
          
          {/* Nova typing indicator for final message (after Michael's response) */}
          {novaTyping && showSecondMichaelMessage && !showFinalMessage && (
            <div className="message flex justify-start">
              <div className="w-8 h-8 rounded-full bg-indigo-600 flex items-center justify-center mr-2 self-end">
                <span className="text-white text-xs font-bold">N</span>
              </div>
              <div className="message-content">
                <div className="message-bubble bg-indigo-900/40 rounded-tr-xl rounded-br-xl rounded-bl-xl p-3 mt-1 flex items-center shadow-md shadow-indigo-900/10">
                  <div className="typing-indicator flex gap-1">
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full"></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.2s" }}></span>
                    <span className="dot animate-bounce bg-gray-400 h-2 w-2 rounded-full" style={{ animationDelay: "0.4s" }}></span>
                  </div>
                </div>
              </div>
            </div>
          )}
          
          {/* Final Nova message */}
          {showFinalMessage && (
            <div className="message flex justify-start">
              <div className="w-8 h-8 rounded-full bg-indigo-600 flex items-center justify-center mr-2 self-end">
                <span className="text-white text-xs font-bold">N</span>
              </div>
              <div className="message-content max-w-[75%]">
                <div 
                  className={`message-bubble bg-indigo-900/40 p-3 rounded-tr-xl rounded-br-xl rounded-bl-xl mt-1 transition-all duration-300 shadow-md shadow-indigo-900/10 ${highlightStory ? 'bg-indigo-700/50 border border-indigo-400/50' : ''}`}
                >
                  {finalMessage.content}
                </div>
              </div>
            </div>
          )}
        </div>
        
        {/* Input area */}
        <div className="chat-footer p-3 border-t border-indigo-900/50">
          <div className="flex items-center gap-2 opacity-50">
            <div className="h-9 w-full bg-gray-800/80 rounded-full"></div>
            <div className="h-9 w-9 bg-indigo-600 rounded-full flex items-center justify-center">
              <svg xmlns="http://www.w3.org/2000/svg" width="18" height="18" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="text-white">
                <path d="M22 2L11 13M22 2l-7 20-4-9-9-4 20-7z"></path>
              </svg>
            </div>
          </div>
        </div>
      </div>
    </div>
  );
}

// Wrap with memo to prevent unnecessary re-renders
// Using a separate variable to avoid ESLint error with anonymous functions
const MemoizedChatDemo = memo(ChatDemo);
export default MemoizedChatDemo;
